@@ -48,6 +48,35 @@ def CHUNK_SIZE():
     return int(float(os.getenv('DATALAKE_CHUNK_SIZE_MB', 100)) * MB_B)
 
 
+_connect_timeout = None
+_read_timeout = None
+
+
+def TIMEOUT():
+    """
+    A tuple of the connect and read timeout, with defaults using the
+    recommendations from the requests docs:
+     https://requests.readthedocs.io/en/master/user/advanced/#timeouts
+
+    :return: tuple of floats
+    """
+    return CONNECT_TIMEOUT(), READ_TIMEOUT(),
+
+
+def CONNECT_TIMEOUT():
+    global _connect_timeout
+    if _connect_timeout is None:
+        _connect_timeout = float(os.getenv('DATALAKE_CONNECT_TIMEOUT_S', 3.05))
+    return _connect_timeout
+
+
+def READ_TIMEOUT():
+    global _read_timeout
+    if _read_timeout is None:
+        _read_timeout = float(os.getenv('DATALAKE_READ_TIMEOUT_S', 31))
+    return _read_timeout
+
+
 class UnsupportedStorageError(Exception):
     pass
 
@@ -62,10 +91,11 @@ class InvalidDatalakePath(Exception):
 
 class Archive(object):
 
-    def __init__(self, storage_url=None, http_url=None):
+    def __init__(self, storage_url=None, http_url=None, session=None):
         self.storage_url = storage_url or environ.get('DATALAKE_STORAGE_URL')
         self._validate_storage_url()
         self._http_url = http_url
+        self.__session = session
 
     def _validate_storage_url(self):
         if not self.storage_url:
@@ -412,8 +442,22 @@ class Archive(object):
         return self._conn
 
     def _requests_get(self, url, **kwargs):
-        return self._session.get(url, **kwargs)
+        return self._session.get(url, timeout=TIMEOUT(), **kwargs)
 
-    @memoized_property
+    @property
     def _session(self):
-        return requests.Session()
+        if self.__session:
+            return self.__session
+
+        session_class = os.environ.get('DATALAKE_SESSION_CLASS')
+        if session_class is not None:
+            import importlib
+            parts = session_class.split('.')
+            module_name = '.'.join(parts[:-1])
+            class_name = parts[-1]
+            module = importlib.import_module(module_name)
+            session_class = getattr(module, class_name)
+            self.__session = session_class()
+        else:
+            self.__session = requests.Session()
+        return self.__session
